@@ -21,8 +21,6 @@
 
 #include "gtkcellrendereraccel.h"
 
-#include "gdk/gdkkeysyms.h"
-
 #include "gtkintl.h"
 #include "gtkaccelgroup.h"
 #include "gtkmarshalers.h"
@@ -30,6 +28,7 @@
 #include "gtkeventbox.h"
 #include "gtkmain.h"
 #include "gtksizerequest.h"
+#include "gtktypebuiltins.h"
 #include "gtkprivate.h"
 
 
@@ -168,14 +167,14 @@ gtk_cell_renderer_accel_class_init (GtkCellRendererAccelClass *cell_accel_class)
    * Since: 2.10
    */ 
   g_object_class_install_property (object_class,
-		  		   PROP_KEYCODE,
-				   g_param_spec_uint ("keycode",
-					   	      P_("Accelerator keycode"),
-						      P_("The hardware keycode of the accelerator"),
-						      0,
-						      G_MAXINT,
-						      0,
-						      GTK_PARAM_READWRITE));
+                                   PROP_KEYCODE,
+                                   g_param_spec_uint ("keycode",
+                                                      P_("Accelerator keycode"),
+                                                      P_("The hardware keycode of the accelerator"),
+                                                      0,
+                                                      G_MAXINT,
+                                                      0,
+                                                      GTK_PARAM_READWRITE));
 
   /**
    * GtkCellRendererAccel:accel-mode:
@@ -190,11 +189,11 @@ gtk_cell_renderer_accel_class_init (GtkCellRendererAccelClass *cell_accel_class)
   g_object_class_install_property (object_class,
                                    PROP_ACCEL_MODE,
                                    g_param_spec_enum ("accel-mode",
-						      P_("Accelerator Mode"),
-						      P_("The type of accelerators"),
-						      GTK_TYPE_CELL_RENDERER_ACCEL_MODE,
-						      GTK_CELL_RENDERER_ACCEL_MODE_GTK,
-						      GTK_PARAM_READWRITE));
+                                                      P_("Accelerator Mode"),
+                                                      P_("The type of accelerators"),
+                                                      GTK_TYPE_CELL_RENDERER_ACCEL_MODE,
+                                                      GTK_CELL_RENDERER_ACCEL_MODE_GTK,
+                                                      GTK_PARAM_READWRITE));
   
   /**
    * GtkCellRendererAccel::accel-edited:
@@ -209,16 +208,16 @@ gtk_cell_renderer_accel_class_init (GtkCellRendererAccelClass *cell_accel_class)
    * Since: 2.10
    */
   signals[ACCEL_EDITED] = g_signal_new (I_("accel-edited"),
-					GTK_TYPE_CELL_RENDERER_ACCEL,
-					G_SIGNAL_RUN_LAST,
-					G_STRUCT_OFFSET (GtkCellRendererAccelClass, accel_edited),
-					NULL, NULL,
-					_gtk_marshal_VOID__STRING_UINT_FLAGS_UINT,
-					G_TYPE_NONE, 4,
-					G_TYPE_STRING,
-					G_TYPE_UINT,
-					GDK_TYPE_MODIFIER_TYPE,
-					G_TYPE_UINT);
+                                        GTK_TYPE_CELL_RENDERER_ACCEL,
+                                        G_SIGNAL_RUN_LAST,
+                                        G_STRUCT_OFFSET (GtkCellRendererAccelClass, accel_edited),
+                                        NULL, NULL,
+                                        _gtk_marshal_VOID__STRING_UINT_FLAGS_UINT,
+                                        G_TYPE_NONE, 4,
+                                        G_TYPE_STRING,
+                                        G_TYPE_UINT,
+                                        GDK_TYPE_MODIFIER_TYPE,
+                                        G_TYPE_UINT);
 
   /**
    * GtkCellRendererAccel::accel-cleared:
@@ -230,13 +229,13 @@ gtk_cell_renderer_accel_class_init (GtkCellRendererAccelClass *cell_accel_class)
    * Since: 2.10
    */
   signals[ACCEL_CLEARED] = g_signal_new (I_("accel-cleared"),
-					 GTK_TYPE_CELL_RENDERER_ACCEL,
-					 G_SIGNAL_RUN_LAST,
-					 G_STRUCT_OFFSET (GtkCellRendererAccelClass, accel_cleared),
-					 NULL, NULL,
-					 g_cclosure_marshal_VOID__STRING,
-					 G_TYPE_NONE, 1,
-					 G_TYPE_STRING);
+                                         GTK_TYPE_CELL_RENDERER_ACCEL,
+                                         G_SIGNAL_RUN_LAST,
+                                         G_STRUCT_OFFSET (GtkCellRendererAccelClass, accel_cleared),
+                                         NULL, NULL,
+                                         g_cclosure_marshal_VOID__STRING,
+                                         G_TYPE_NONE, 1,
+                                         G_TYPE_STRING);
 
   g_type_class_add_private (cell_accel_class, sizeof (GtkCellRendererAccelPrivate));
 }
@@ -259,9 +258,9 @@ gtk_cell_renderer_accel_new (void)
 
 static gchar *
 convert_keysym_state_to_string (GtkCellRendererAccel *accel,
-				guint                 keysym,
+                                guint                 keysym,
                                 GdkModifierType       mask,
-				guint                 keycode)
+                                guint                 keycode)
 {
   GtkCellRendererAccelPrivate *priv = accel->priv;
 
@@ -284,24 +283,24 @@ convert_keysym_state_to_string (GtkCellRendererAccel *accel,
           return gtk_accelerator_get_label (keysym, mask);
         }
       else 
-	{
-	  gchar *name;
+        {
+          gchar *name;
 
-	  name = gtk_accelerator_get_label (keysym, mask);
-	  if (name == NULL)
-	    name = gtk_accelerator_name (keysym, mask);
+          name = gtk_accelerator_get_label (keysym, mask);
+          if (name == NULL)
+            name = gtk_accelerator_name (keysym, mask);
 
-	  if (keysym == 0)
-	    {
-	      gchar *tmp;
+          if (keysym == 0)
+            {
+              gchar *tmp;
 
-	      tmp = name;
-	      name = g_strdup_printf ("%s0x%02x", tmp, keycode);
-	      g_free (tmp);
-	    }
+              tmp = name;
+              name = g_strdup_printf ("%s0x%02x", tmp, keycode);
+              g_free (tmp);
+            }
 
-	  return name;
-	}
+          return name;
+        }
     }
 }
 
@@ -350,36 +349,36 @@ gtk_cell_renderer_accel_set_property  (GObject      *object,
     {
     case PROP_ACCEL_KEY:
       {
-	guint accel_key = g_value_get_uint (value);
+        guint accel_key = g_value_get_uint (value);
 
-	if (priv->accel_key != accel_key)
-	  {
-	    priv->accel_key = accel_key;
-	    changed = TRUE;
-	  }
+        if (priv->accel_key != accel_key)
+          {
+            priv->accel_key = accel_key;
+            changed = TRUE;
+          }
       }
       break;
 
     case PROP_ACCEL_MODS:
       {
-	guint accel_mods = g_value_get_flags (value);
+        guint accel_mods = g_value_get_flags (value);
 
-	if (priv->accel_mods != accel_mods)
-	  {
-	    priv->accel_mods = accel_mods;
-	    changed = TRUE;
-	  }
+        if (priv->accel_mods != accel_mods)
+          {
+            priv->accel_mods = accel_mods;
+            changed = TRUE;
+          }
       }
       break;
     case PROP_KEYCODE:
       {
-	guint keycode = g_value_get_uint (value);
+        guint keycode = g_value_get_uint (value);
 
-	if (priv->keycode != keycode)
-	  {
-	    priv->keycode = keycode;
-	    changed = TRUE;
-	  }
+        if (priv->keycode != keycode)
+          {
+            priv->keycode = keycode;
+            changed = TRUE;
+          }
       }
       break;
 
@@ -452,10 +451,10 @@ grab_key_callback (GtkWidget            *widget,
   cleared = FALSE;
 
   gdk_keymap_translate_keyboard_state (gdk_keymap_get_for_display (display),
-				       event->hardware_keycode,
+                                       event->hardware_keycode,
                                        event->state,
                                        event->group,
-				       NULL, NULL, NULL, &consumed_modifiers);
+                                       NULL, NULL, NULL, &consumed_modifiers);
 
   accel_key = gdk_keyval_to_lower (event->keyval);
   if (accel_key == GDK_KEY_ISO_Left_Tab) 
@@ -476,26 +475,26 @@ grab_key_callback (GtkWidget            *widget,
   if (accel_mods == 0)
     {
       switch (event->keyval)
-	{
-	case GDK_KEY_Escape:
-	  goto out; /* cancel */
-	case GDK_KEY_BackSpace:
-	  /* clear the accelerator on Backspace */
-	  cleared = TRUE;
-	  goto out;
-	default:
-	  break;
-	}
+        {
+        case GDK_KEY_Escape:
+          goto out; /* cancel */
+        case GDK_KEY_BackSpace:
+          /* clear the accelerator on Backspace */
+          cleared = TRUE;
+          goto out;
+        default:
+          break;
+        }
     }
 
   if (priv->accel_mode == GTK_CELL_RENDERER_ACCEL_MODE_GTK)
     {
       if (!gtk_accelerator_valid (accel_key, accel_mods))
-	{
-	  gtk_widget_error_bell (widget);
+        {
+          gtk_widget_error_bell (widget);
 
-	  return TRUE;
-	}
+          return TRUE;
+        }
     }
 
   edited = TRUE;
@@ -516,7 +515,7 @@ grab_key_callback (GtkWidget            *widget,
 
   if (edited)
     g_signal_emit (accel, signals[ACCEL_EDITED], 0, path, 
-		   accel_key, accel_mods, event->hardware_keycode);
+                   accel_key, accel_mods, event->hardware_keycode);
   else if (cleared)
     g_signal_emit (accel, signals[ACCEL_CLEARED], 0, path);
 
@@ -545,7 +544,7 @@ ungrab_stuff (GtkWidget            *widget,
 
 static void
 _gtk_cell_editable_event_box_start_editing (GtkCellEditable *cell_editable,
-					    GdkEvent        *event)
+                                            GdkEvent        *event)
 {
   /* do nothing, because we are pointless */
 }
@@ -556,17 +555,73 @@ _gtk_cell_editable_event_box_cell_editable_init (GtkCellEditableIface *iface)
   iface->start_editing = _gtk_cell_editable_event_box_start_editing;
 }
 
-typedef GtkEventBox      GtkCellEditableEventBox;
-typedef GtkEventBoxClass GtkCellEditableEventBoxClass;
+typedef struct _GtkCellEditableEventBox GtkCellEditableEventBox;
+typedef         GtkEventBoxClass        GtkCellEditableEventBoxClass;
+
+struct _GtkCellEditableEventBox
+{
+  GtkEventBox box;
+  gboolean editing_canceled;
+};
 
 G_DEFINE_TYPE_WITH_CODE (GtkCellEditableEventBox, _gtk_cell_editable_event_box, GTK_TYPE_EVENT_BOX, { \
     G_IMPLEMENT_INTERFACE (GTK_TYPE_CELL_EDITABLE, _gtk_cell_editable_event_box_cell_editable_init)   \
       })
 
+enum {
+  PROP_ZERO,
+  PROP_EDITING_CANCELED
+};
+
+static void
+gtk_cell_editable_event_box_set_property (GObject      *object,
+                                          guint         prop_id,
+                                          const GValue *value,
+                                          GParamSpec   *pspec)
+{
+  GtkCellEditableEventBox *box = (GtkCellEditableEventBox*)object;
+
+  switch (prop_id)
+    {
+    case PROP_EDITING_CANCELED:
+      box->editing_canceled = g_value_get_boolean (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+gtk_cell_editable_event_box_get_property (GObject    *object,
+                                          guint       prop_id,
+                                          GValue     *value,
+                                          GParamSpec *pspec)
+{
+  GtkCellEditableEventBox *box = (GtkCellEditableEventBox*)object;
+
+  switch (prop_id)
+    {
+    case PROP_EDITING_CANCELED:
+      g_value_set_boolean (value, box->editing_canceled);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
 
 static void
 _gtk_cell_editable_event_box_class_init (GtkCellEditableEventBoxClass *class)
 {
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+
+  gobject_class->set_property = gtk_cell_editable_event_box_set_property;
+  gobject_class->get_property = gtk_cell_editable_event_box_get_property;
+
+  g_object_class_override_property (gobject_class,
+                                    PROP_EDITING_CANCELED,
+                                    "editing-canceled");
 }
 
 static void
@@ -586,7 +641,8 @@ gtk_cell_renderer_accel_start_editing (GtkCellRenderer      *cell,
   GtkCellRendererAccelPrivate *priv;
   GtkCellRendererText *celltext;
   GtkCellRendererAccel *accel;
-  GtkStyle *style;
+  GtkStyleContext *context;
+  GdkRGBA color;
   GtkWidget *label;
   GtkWidget *eventbox;
   GdkDevice *device, *keyb, *pointer;
@@ -604,7 +660,7 @@ gtk_cell_renderer_accel_start_editing (GtkCellRenderer      *cell,
     return NULL;
 
   window = gtk_widget_get_window (widget);
-  style = gtk_widget_get_style (widget);
+  context = gtk_widget_get_style_context (widget);
 
   g_return_val_if_fail (window != NULL, NULL);
 
@@ -660,13 +716,11 @@ gtk_cell_renderer_accel_start_editing (GtkCellRenderer      *cell,
   label = gtk_label_new (NULL);
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 
-  
+  gtk_style_context_get_background_color (context, GTK_STATE_FLAG_SELECTED, &color);
+  gtk_widget_override_background_color (label, 0, &color);
 
-  gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL,
-                        &style->bg[GTK_STATE_SELECTED]);
-
-  gtk_widget_modify_fg (label, GTK_STATE_NORMAL,
-                        &style->fg[GTK_STATE_SELECTED]);
+  gtk_style_context_get_color (context, GTK_STATE_FLAG_SELECTED, &color);
+  gtk_widget_override_color (label, 0, &color);
 
   /* This label is displayed in a treeview cell displaying
    * an accelerator when the cell is clicked to change the 
