@@ -47,6 +47,8 @@
 #include "gtkprivate.h"
 #include "gtktoggleaction.h"
 #include "gtkwidget.h"
+#include "gtkmarshalers.h"
+
 
 #define DEFAULT_SLIDER_WIDTH    (36)
 #define DEFAULT_SLIDER_HEIGHT   (22)
@@ -77,6 +79,14 @@ enum
   LAST_PROP
 };
 
+enum
+{
+  ACTIVATE,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static GParamSpec *switch_props[LAST_PROP] = { NULL, };
 
 static GType gtk_switch_accessible_factory_get_type (void);
@@ -152,17 +162,30 @@ gtk_switch_motion (GtkWidget      *widget,
       GtkStyleContext *context;
       GtkStateFlags state;
       GtkBorder padding;
+      gint width, focus_width, focus_pad;
+
+      gtk_widget_style_get (widget,
+                            "focus-line-width", &focus_width,
+                            "focus-padding", &focus_pad,
+                            NULL);
 
       context = gtk_widget_get_style_context (widget);
       state = gtk_widget_get_state_flags (widget);
+
+      gtk_style_context_save (context);
+      gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
       gtk_style_context_get_padding (context, state, &padding);
+      gtk_style_context_restore (context);
+      
       gtk_widget_get_allocation (widget, &allocation);
 
+      width = allocation.width - 2 * (focus_width + focus_pad);
+
       /* constrain the handle within the trough width */
-      if (position > (allocation.width / 2 - padding.right))
-        priv->handle_x = allocation.width / 2 - padding.right;
+      if (position > (width / 2) - padding.right)
+        priv->handle_x = width / 2 - padding.right;
       else if (position < padding.left)
-        priv->handle_x = padding.left;
+        priv->handle_x = 0;
       else
         priv->handle_x = position;
 
@@ -262,22 +285,12 @@ gtk_switch_leave (GtkWidget        *widget,
   return FALSE;
 }
 
-static gboolean
-gtk_switch_key_release (GtkWidget   *widget,
-                        GdkEventKey *event)
+static void
+gtk_switch_activate (GtkSwitch *sw)
 {
-  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
+  GtkSwitchPrivate *priv = sw->priv;
 
-  if (event->keyval == GDK_KEY_Return ||
-      event->keyval == GDK_KEY_KP_Enter ||
-      event->keyval == GDK_KEY_ISO_Enter ||
-      event->keyval == GDK_KEY_space ||
-      event->keyval == GDK_KEY_KP_Space)
-    {
-      gtk_switch_set_active (GTK_SWITCH (widget), !priv->is_active);
-    }
-
-  return FALSE;
+  gtk_switch_set_active (sw, !priv->is_active);
 }
 
 static void
@@ -285,6 +298,7 @@ gtk_switch_get_preferred_width (GtkWidget *widget,
                                 gint      *minimum,
                                 gint      *natural)
 {
+  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
   GtkStyleContext *context;
   GtkStateFlags state;
   GtkBorder padding;
@@ -294,10 +308,20 @@ gtk_switch_get_preferred_width (GtkWidget *widget,
 
   context = gtk_widget_get_style_context (widget);
   state = gtk_widget_get_state_flags (widget);
+
+  if (priv->is_active)
+    state |= GTK_STATE_FLAG_ACTIVE;
+
+  gtk_style_context_save (context);
+
+  gtk_style_context_set_state (context, state);
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
   gtk_style_context_get_padding (context, state, &padding);
 
   width = padding.left + padding.right;
 
+  gtk_style_context_restore (context);
+
   gtk_widget_style_get (widget,
                         "slider-width", &slider_width,
                         "focus-line-width", &focus_width,
@@ -337,6 +361,7 @@ gtk_switch_get_preferred_height (GtkWidget *widget,
                                  gint      *minimum,
                                  gint      *natural)
 {
+  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
   GtkStyleContext *context;
   GtkStateFlags state;
   GtkBorder padding;
@@ -347,10 +372,20 @@ gtk_switch_get_preferred_height (GtkWidget *widget,
 
   context = gtk_widget_get_style_context (widget);
   state = gtk_widget_get_state_flags (widget);
+
+  if (priv->is_active)
+    state |= GTK_STATE_FLAG_ACTIVE;
+
+  gtk_style_context_save (context);
+
+  gtk_style_context_set_state (context, state);
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
   gtk_style_context_get_padding (context, state, &padding);
 
   height = padding.top + padding.bottom;
 
+  gtk_style_context_restore (context);
+
   gtk_widget_style_get (widget,
                         "focus-line-width", &focus_width,
                         "focus-padding", &focus_pad,
@@ -473,11 +508,15 @@ gtk_switch_paint_handle (GtkWidget    *widget,
                          cairo_t      *cr,
                          GdkRectangle *box)
 {
+  GtkSwitchPrivate *priv = GTK_SWITCH (widget)->priv;
   GtkStyleContext *context = gtk_widget_get_style_context (widget);
   GtkStateFlags state;
 
   state = gtk_widget_get_state_flags (widget);
 
+  if (priv->is_active)
+    state |= GTK_STATE_FLAG_ACTIVE;
+
   gtk_style_context_save (context);
   gtk_style_context_set_state (context, state);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
@@ -516,8 +555,15 @@ gtk_switch_draw (GtkWidget *widget,
   if (priv->is_active)
     state |= GTK_STATE_FLAG_ACTIVE;
 
+  gtk_style_context_save (context);
+
+  gtk_style_context_set_state (context, state);
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_SLIDER);
+
   gtk_style_context_get_padding (context, state, &padding);
 
+  gtk_style_context_restore (context);
+
   x = 0;
   y = 0;
   width = gtk_widget_get_allocated_width (widget);
@@ -526,25 +572,27 @@ gtk_switch_draw (GtkWidget *widget,
   if (gtk_widget_has_focus (widget))
     gtk_render_focus (context, cr, x, y, width, height);
 
-  gtk_style_context_save (context);
-  gtk_style_context_set_state (context, state);
-
   x += focus_width + focus_pad;
   y += focus_width + focus_pad;
   width -= 2 * (focus_width + focus_pad);
   height -= 2 * (focus_width + focus_pad);
 
+  gtk_style_context_save (context);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_TROUGH);
+  gtk_style_context_set_state (context, state);
 
   gtk_render_background (context, cr, x, y, width, height);
   gtk_render_frame (context, cr, x, y, width, height);
 
-  /* XXX the +1/-1 it's pixel wriggling after checking with the default
-   * theme and xmag
-   */
-  handle.y = y + padding.top + 1;
-  handle.width = (width - padding.left - padding.right) / 2;
-  handle.height = (height - padding.top - padding.bottom) - 1;
+  width -= padding.left + padding.right;
+  height -= padding.top + padding.bottom;
+
+  x += padding.left;
+  y += padding.top;
+
+  handle.y = y;
+  handle.width = width / 2;
+  handle.height = height;
 
   /* Translators: if the "on" state label requires more than three
    * glyphs then use MEDIUM VERTICAL BAR (U+2759) as the text for
@@ -554,10 +602,8 @@ gtk_switch_draw (GtkWidget *widget,
   pango_layout_get_extents (layout, NULL, &rect);
   pango_extents_to_pixels (&rect, NULL);
 
-  label_x = x + padding.left
-          + ((width / 2) - rect.width - padding.left - padding.right) / 2;
-  label_y = y + padding.top
-          + (height - rect.height - padding.top - padding.bottom) / 2;
+  label_x = x +  ((width / 2) - rect.width) / 2;
+  label_y = y + (height - rect.height) / 2;
 
   gtk_render_layout (context, cr, label_x, label_y, layout);
 
@@ -570,11 +616,8 @@ gtk_switch_draw (GtkWidget *widget,
   pango_layout_get_extents (layout, NULL, &rect);
   pango_extents_to_pixels (&rect, NULL);
 
-  label_x = x + padding.left
-          + (width / 2)
-          + ((width / 2) - rect.width - padding.left - padding.right) / 2;
-  label_y = y + padding.top
-          + (height - rect.height - padding.top - padding.bottom) / 2;
+  label_x = x +  (width / 2) + ((width / 2) - rect.width) / 2;
+  label_y = y + (height - rect.height) / 2;
 
   gtk_render_layout (context, cr, label_x, label_y, layout);
 
@@ -583,9 +626,9 @@ gtk_switch_draw (GtkWidget *widget,
   if (priv->is_dragging)
     handle.x = x + priv->handle_x;
   else if (priv->is_active)
-    handle.x = x + width - handle.width - padding.right;
+    handle.x = x + width - handle.width;
   else
-    handle.x = x + padding.left;
+    handle.x = x;
 
   gtk_style_context_restore (context);
 
@@ -756,9 +799,10 @@ gtk_switch_class_init (GtkSwitchClass *klass)
   widget_class->motion_notify_event = gtk_switch_motion;
   widget_class->enter_notify_event = gtk_switch_enter;
   widget_class->leave_notify_event = gtk_switch_leave;
-  widget_class->key_release_event = gtk_switch_key_release;
   widget_class->get_accessible = gtk_switch_get_accessible;
 
+  klass->activate = gtk_switch_activate;
+
   /**
    * GtkSwitch:slider-width:
    *
@@ -771,6 +815,26 @@ gtk_switch_class_init (GtkSwitchClass *klass)
                                                              DEFAULT_SLIDER_WIDTH, G_MAXINT,
                                                              DEFAULT_SLIDER_WIDTH,
                                                              GTK_PARAM_READABLE));
+
+  /**
+   * GtkSwitch::activate:
+   * @widget: the object which received the signal.
+   *
+   * The ::activate signal on GtkSwitch is an action signal and
+   * emitting it causes the switch to animate.
+   * Applications should never connect to this signal, but use the
+   * notify::active signal.
+   */
+  signals[ACTIVATE] =
+    g_signal_new (I_("activate"),
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+                  G_STRUCT_OFFSET (GtkSwitchClass, activate),
+                  NULL, NULL,
+                  _gtk_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+  widget_class->activate_signal = signals[ACTIVATE];
+
 }
 
 static void
