@@ -155,7 +155,7 @@ static void       gtk_tooltip_class_init           (GtkTooltipClass *klass);
 static void       gtk_tooltip_init                 (GtkTooltip      *tooltip);
 static void       gtk_tooltip_dispose              (GObject         *object);
 
-static void       gtk_tooltip_window_style_set     (GtkTooltip      *tooltip);
+static void       gtk_tooltip_window_style_updated (GtkTooltip      *tooltip);
 static gboolean   gtk_tooltip_paint_window         (GtkTooltip      *tooltip,
                                                     cairo_t         *cr);
 static void       gtk_tooltip_window_hide          (GtkWidget       *widget,
@@ -182,7 +182,8 @@ gtk_tooltip_class_init (GtkTooltipClass *klass)
 static void
 gtk_tooltip_init (GtkTooltip *tooltip)
 {
-  GtkStyle *style;
+  GtkStyleContext *context;
+  GtkBorder padding, border;
 
   tooltip->timeout_id = 0;
   tooltip->browse_mode_timeout_id = 0;
@@ -207,21 +208,27 @@ gtk_tooltip_init (GtkTooltip *tooltip)
   g_signal_connect (tooltip->window, "hide",
 		    G_CALLBACK (gtk_tooltip_window_hide), tooltip);
 
-  style = gtk_widget_get_style (tooltip->window);
+  context = gtk_widget_get_style_context (tooltip->window);
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_TOOLTIP);
+
+  gtk_style_context_get_padding (context, 0, &padding);
+  gtk_style_context_get_border (context, 0, &border);
 
   tooltip->alignment = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
   gtk_alignment_set_padding (GTK_ALIGNMENT (tooltip->alignment),
-                             style->ythickness, style->ythickness,
-                             style->xthickness, style->xthickness);
+                             border.top + padding.top,
+                             border.bottom + padding.bottom,
+                             border.left + padding.left,
+                             border.right + padding.right);
   gtk_container_add (GTK_CONTAINER (tooltip->window), tooltip->alignment);
   gtk_widget_show (tooltip->alignment);
 
-  g_signal_connect_swapped (tooltip->window, "style-set",
-			    G_CALLBACK (gtk_tooltip_window_style_set), tooltip);
+  g_signal_connect_swapped (tooltip->window, "style-updated",
+			    G_CALLBACK (gtk_tooltip_window_style_updated), tooltip);
   g_signal_connect_swapped (tooltip->window, "draw",
 			    G_CALLBACK (gtk_tooltip_paint_window), tooltip);
 
-  tooltip->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, style->xthickness);
+  tooltip->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, padding.left);
   gtk_container_add (GTK_CONTAINER (tooltip->alignment), tooltip->box);
   gtk_widget_show (tooltip->box);
 
@@ -532,9 +539,11 @@ gtk_tooltip_trigger_tooltip_query (GdkDisplay *display)
   gint x, y;
   GdkWindow *window;
   GdkEvent event;
+  GdkDevice *device;
 
   /* Trigger logic as if the mouse moved */
-  window = gdk_display_get_window_at_pointer (display, &x, &y);
+  device = gdk_device_manager_get_client_pointer (gdk_display_get_device_manager (display));
+  window = gdk_device_get_window_at_position (device, &x, &y);
   if (!window)
     return;
 
@@ -567,18 +576,22 @@ gtk_tooltip_reset (GtkTooltip *tooltip)
 }
 
 static void
-gtk_tooltip_window_style_set (GtkTooltip *tooltip)
+gtk_tooltip_window_style_updated (GtkTooltip *tooltip)
 {
-  GtkStyle *style;
+  GtkStyleContext *context;
+  GtkBorder padding, border;
 
-  style = gtk_widget_get_style (tooltip->window);
+  context = gtk_widget_get_style_context (tooltip->window);
+  gtk_style_context_get_padding (context, 0, &padding);
+  gtk_style_context_get_border (context, 0, &border);
 
   gtk_alignment_set_padding (GTK_ALIGNMENT (tooltip->alignment),
-                             style->ythickness, style->ythickness,
-                             style->xthickness, style->xthickness);
+                             border.top + padding.top,
+                             border.bottom + padding.bottom,
+                             border.left + padding.left,
+                             border.right + padding.right);
 
-  gtk_box_set_spacing (GTK_BOX (tooltip->box),
-                       style->xthickness);
+  gtk_box_set_spacing (GTK_BOX (tooltip->box), padding.left);
 
   gtk_widget_queue_draw (tooltip->window);
 }
@@ -587,15 +600,16 @@ static gboolean
 gtk_tooltip_paint_window (GtkTooltip *tooltip,
                           cairo_t    *cr)
 {
-  gtk_paint_flat_box (gtk_widget_get_style (tooltip->window),
-		      cr,
-		      GTK_STATE_NORMAL,
-		      GTK_SHADOW_OUT,
-		      tooltip->window,
-		      "tooltip",
-		      0, 0,
-                      gtk_widget_get_allocated_width (tooltip->window),
-                      gtk_widget_get_allocated_height (tooltip->window));
+  GtkStyleContext *context;
+
+  context = gtk_widget_get_style_context (tooltip->window);
+
+  gtk_render_background (context, cr, 0, 0,
+			 gtk_widget_get_allocated_width (tooltip->window),
+			 gtk_widget_get_allocated_height (tooltip->window));
+  gtk_render_frame (context, cr, 0, 0,
+                    gtk_widget_get_allocated_width (tooltip->window),
+                    gtk_widget_get_allocated_height (tooltip->window));
 
   return FALSE;
 }
@@ -1146,18 +1160,17 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
 
   GdkWindow *window;
   GtkWidget *tooltip_widget;
-  GtkWidget *pointer_widget;
   GtkTooltip *tooltip;
   gboolean has_tooltip;
   gboolean return_value = FALSE;
 
   tooltip = g_object_get_data (G_OBJECT (display),
-			       "gdk-display-current-tooltip");
+                               "gdk-display-current-tooltip");
 
   if (tooltip->keyboard_mode_enabled)
     {
       x = y = -1;
-      pointer_widget = tooltip_widget = tooltip->keyboard_widget;
+      tooltip_widget = tooltip->keyboard_widget;
     }
   else
     {
@@ -1166,7 +1179,7 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
       window = tooltip->last_window;
 
       if (!GDK_IS_WINDOW (window))
-	return;
+        return;
 
       gdk_window_get_pointer (window, &x, &y, NULL);
 
@@ -1174,9 +1187,7 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
       tooltip->last_x = tx;
       tooltip->last_y = ty;
 
-      pointer_widget = tooltip_widget = _gtk_widget_find_at_coords (window,
-                                                                    x, y,
-                                                                    &x, &y);
+      tooltip_widget = _gtk_widget_find_at_coords (window, x, y, &x, &y);
     }
 
   if (!tooltip_widget)
@@ -1191,9 +1202,9 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
   if (!tooltip->current_window)
     {
       if (gtk_widget_get_tooltip_window (tooltip_widget))
-	tooltip->current_window = gtk_widget_get_tooltip_window (tooltip_widget);
+        tooltip->current_window = gtk_widget_get_tooltip_window (tooltip_widget);
       else
-	tooltip->current_window = GTK_WINDOW (GTK_TOOLTIP (tooltip)->window);
+        tooltip->current_window = GTK_WINDOW (GTK_TOOLTIP (tooltip)->window);
     }
 
   screen = gtk_widget_get_screen (tooltip_widget);
@@ -1202,13 +1213,13 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
   if (screen != gtk_widget_get_screen (tooltip->window))
     {
       g_signal_handlers_disconnect_by_func (display,
-					    gtk_tooltip_display_closed,
-					    tooltip);
+                                            gtk_tooltip_display_closed,
+                                            tooltip);
 
       gtk_window_set_screen (GTK_WINDOW (tooltip->window), screen);
 
       g_signal_connect (display, "closed",
-			G_CALLBACK (gtk_tooltip_display_closed), tooltip);
+                        G_CALLBACK (gtk_tooltip_display_closed), tooltip);
     }
 
   gtk_tooltip_position (tooltip, display, tooltip_widget);
